#ifndef _ASM_ALPHA_SIGNAL_H
#define _ASM_ALPHA_SIGNAL_H

#include <linux/posix_types.h>

#define SIG_BLOCK          1	/* for blocking signals */
#define SIG_UNBLOCK        2	/* for unblocking signals */
#define SIG_SETMASK        3	/* for setting the signal mask */

struct sigaction {
	__kernel_sighandler_t	sa_handler;
	__kernel_sigset_t	sa_mask;
	unsigned int		sa_flags;
};

/*
 * sa_flags values: SA_STACK is not currently supported.  On the x86, the
 * idea is to use the obsolete sa_restorer field in "struct sigaction"
 * as a stack pointer.  Due to OSF/1 compatibility, the Alpha doesn't have
 * such a field, however.
 *
 * SA_INTERRUPT is a no-op, but left due to historical reasons. Use the
 * SA_RESTART flag to get restarting signals (which were the default long ago)
 * SA_SHIRQ flag is for shared interrupt support on PCI and EISA.
 */
#define __KERNEL_SA_STACK	0x00000001
#define __KERNEL_SA_RESTART	0x00000002
#define __KERNEL_SA_NOCLDSTOP	0x00000004
#define __KERNEL_SA_INTERRUPT	0x20000000
#define __KERNEL_SA_NOMASK	0x00000008
#define __KERNEL_SA_ONESHOT	0x00000010
#define __KERNEL_SA_SHIRQ	0x00000020

#ifdef __KERNEL__
/*
 * These values of sa_flags are used only by the kernel as part of the
 * irq handling routines.
 *
 * SA_INTERRUPT is also used by the irq handling routines.
 */
# define SA_PROBE		SA_ONESHOT
# define SA_SAMPLE_RANDOM	SA_RESTART

# include <asm/sigcontext.h>
#endif

#ifndef __KERNEL_STRICT_NAMES
# include <asm/signum.h>

  typedef __kernel_sighandler_t	__sighandler_t;
  typedef __kernel_sigset_t	sigset_t;

# define SA_STACK		__KERNEL_SA_STACK
# define SA_RESTART		__KERNEL_SA_RESTART
# define SA_NOCLDSTOP		__KERNEL_SA_NOCLDSTOP
# define SA_INTERRUPT		__KERNEL_SA_INTERRUPT
# define SA_NOMASK		__KERNEL_SA_NOMASK
# define SA_ONESHOT		__KERNEL_SA_ONESHOT
# define SA_SHIRQ		__KERNEL_SA_SHIRQ
#endif

#endif /* _ASM_ALPHA_SIGNAL_H */
