#ifndef _LINUX_IPC_H
#define _LINUX_IPC_H

#include <linux/types.h>
#include <asm/ipcbits.h>

/*
 * For those Linux architectures that multiplex the ipc functions
 * through a single system call, these are the function numbers.
 */
#define __KERNEL_IPC_SEMOP	 	1
#define __KERNEL_IPC_SEMGET 		2
#define __KERNEL_IPC_SEMCTL 		3
#define __KERNEL_IPC_MSGSND 		11
#define __KERNEL_IPC_MSGRCV 		12
#define __KERNEL_IPC_MSGGET 		13
#define __KERNEL_IPC_MSGCTL 		14
#define __KERNEL_IPC_SHMAT 		21
#define __KERNEL_IPC_SHMDT 		22
#define __KERNEL_IPC_SHMGET 		23
#define __KERNEL_IPC_SHMCTL 		24

#ifndef __KERNEL_STRICT_NAMES
# define IPC_PRIVATE	__KERNEL_IPC_PRIVATE
# define IPC_CREAT	__KERNEL_IPC_CREAT
# define IPC_EXCL	__KERNEL_IPC_EXCL
# define IPC_NOWAIT	__KERNEL_IPC_NOWAIT
# define IPC_RMID	__KERNEL_IPC_RMID
# define IPC_SET	__KERNEL_IPC_SET
# define IPC_STAT	__KERNEL_IPC_STAT
# define IPC_INFO	__KERNEL_IPC_INFO
#endif

#ifdef __KERNEL__

/* special shmsegs[id], msgque[id] or semary[id]  values */
#define IPC_UNUSED	((void *) -1)
#define IPC_NOID	((void *) -2)		/* being allocated/destroyed */

/* 
 * These are used to wrap system calls. See ipc/util.c.
 */
struct ipc_kludge {
    struct msgbuf *msgp;
    long msgtyp;
};

#define SEMOP	 	__KERNEL_IPC_SEMOP
#define SEMGET 		__KERNEL_IPC_SEMGET
#define SEMCTL 		__KERNEL_IPC_SEMCTL
#define MSGSND 		__KERNEL_IPC_MSGSND
#define MSGRCV 		__KERNEL_IPC_MSGRCV
#define MSGGET 		__KERNEL_IPC_MSGGET
#define MSGCTL 		__KERNEL_IPC_MSGCTL
#define SHMAT 		__KERNEL_IPC_SHMAT
#define SHMDT 		__KERNEL_IPC_SHMDT
#define SHMGET 		__KERNEL_IPC_SHMGET
#define SHMCTL 		__KERNEL_IPC_SHMCTL

#define IPCCALL(version,op)	((version)<<16 | (op))

#endif /* __KERNEL__ */

#endif /* _LINUX_IPC_H */
