#ifndef _LINUX_MSG_H
#define _LINUX_MSG_H

#include <linux/ipc.h>
#include <asm/msgbits.h>

#ifndef __KERNEL_STRICT_NAMES
# include <asm/msginfobits.h>
#endif

/* max # of msg queue identifiers */
#define __KERNEL_MSGMNI   128		/* <= 1K */
/* max size of message (bytes) */
#define __KERNEL_MSGMAX  4056		/* <= 4056 */
/* default max size of a message queue */
#define __KERNEL_MSGMNB 16384		/* ? */

/* unused */
/* size in kilobytes of message pool */
#define __KERNEL_MSGPOOL	(__KERNEL_MSGMNI*__KERNEL_MSGMNB/1024) 
/* number of system message headers */
#define __KERNEL_MSGTQL		__KERNEL_MSGMNB
/* number of entries in message map */
#define __KERNEL_MSGMAP		__KERNEL_MSGMNB           
/* message segment size */
#define __KERNEL_MSGSSZ		16
/* max no. of segments */
#define __KERNEL__MSGSEG	((__KERNEL_MSGPOOL*1024) / __KERNEL_MSGSSZ)
#define __KERNEL_MSGSEG \
	(__KERNEL__MSGSEG <= 0xffff ? __KERNEL__MSGSEG : 0xffff)

#ifndef __KERNEL_STRICT_NAMES
# define MSG_NOERROR	__KERNEL_MSG_NOERROR
# define MSG_EXCEPT	__KERNEL_MSG_EXCEPT
# define MSGMNI		__KERNEL_MSGMNI
# define MSGMAX		__KERNEL_MSGMAX
# define MSGMNB		__KERNEL_MSGMNB
# define MSGPOOL	__KERNEL_MSGPOOL
# define MSGTQL		__KERNEL_MSGTQL
# define MSGMAP		__KERNEL_MSGMAP
# define MSGSSZ		__KERNEL_MSGSSZ
# define MSGSEG		__KERNEL_MSGSEG
# define MSG_STAT	__KERNEL_MSG_STAT
# define MSG_INFO	__KERNEL_MSG_INFO
#endif

#ifdef __KERNEL__

/* one msg structure for each message */
struct msg {
    struct msg *msg_next;   /* next message on queue */
    long  msg_type;          
    char *msg_spot;         /* message text address */
    time_t msg_stime;       /* msgsnd time */
    short msg_ts;           /* message text size */
};

asmlinkage int sys_msgget (key_t key, int msgflg);
asmlinkage int sys_msgsnd (int msqid, struct msgbuf *msgp, size_t msgsz, int msgflg);
asmlinkage int sys_msgrcv (int msqid, struct msgbuf *msgp, size_t msgsz, long msgtyp, int msgflg);
asmlinkage int sys_msgctl (int msqid, int cmd, struct msqid_ds *buf);

#endif /* __KERNEL__ */

#endif /* _LINUX_MSG_H */
