#ifndef _LINUX_RESOURCE_H
#define _LINUX_RESOURCE_H

#include <linux/time.h>

/*
 * Resource control/accounting header file for linux
 */

/*
 * Definition of struct rusage taken from BSD 4.3 Reno
 * 
 * We don't support all of these yet, but we might as well have them....
 * Otherwise, each time we add new items, programs which depend on this
 * structure will lose.  This reduces the chances of that happening.
 */
#define __KERNEL_RUSAGE_SELF		0
#define __KERNEL_RUSAGE_CHILDREN	(-1)
#define __KERNEL_RUSAGE_BOTH		(-2)	/* sys_wait4() uses this */

struct	rusage {
	struct timeval ru_utime;	/* user time used */
	struct timeval ru_stime;	/* system time used */
	long	ru_maxrss;		/* maximum resident set size */
	long	ru_ixrss;		/* integral shared memory size */
	long	ru_idrss;		/* integral unshared data size */
	long	ru_isrss;		/* integral unshared stack size */
	long	ru_minflt;		/* page reclaims */
	long	ru_majflt;		/* page faults */
	long	ru_nswap;		/* swaps */
	long	ru_inblock;		/* block input operations */
	long	ru_oublock;		/* block output operations */
	long	ru_msgsnd;		/* messages sent */
	long	ru_msgrcv;		/* messages received */
	long	ru_nsignals;		/* signals received */
	long	ru_nvcsw;		/* voluntary context switches */
	long	ru_nivcsw;		/* involuntary " */
};

struct rlimit {
	long	rlim_cur;
	long	rlim_max;
};

#define __KERNEL_PRIO_MIN	(-20)
#define __KERNEL_PRIO_MAX	20

#define __KERNEL_PRIO_PROCESS	0
#define __KERNEL_PRIO_PGRP	1
#define __KERNEL_PRIO_USER	2

/*
 * Due to binary compatibility, the actual resource numbers may be
 * different for different linux versions..
 */
#include <asm/resource.h>

#ifndef __KERNEL_RLIM_INFINITY
  /* 32 bit architectures use this value: */
# define __KERNEL_RLIM_INFINITY	0x7fffffff
#endif

#ifndef __KERNEL_STRICT_NAMES
# define RLIMIT_CPU	__KERNEL_RLIMIT_CPU
# define RLIMIT_FSIZE	__KERNEL_RLIMIT_FSIZE
# define RLIMIT_DATA	__KERNEL_RLIMIT_DATA
# define RLIMIT_STACK	__KERNEL_RLIMIT_STACK
# define RLIMIT_CORE	__KERNEL_RLIMIT_CORE
# define RLIMIT_RSS	__KERNEL_RLIMIT_RSS
# define RLIMIT_NOFILE	__KERNEL_RLIMIT_NOFILE
# define RLIMIT_AS	__KERNEL_RLIMIT_AS
# define RLIMIT_NPROC	__KERNEL_RLIMIT_NPROC
# define RLIMIT_MEMLOCK	__KERNEL_RLIMIT_MEMLOCK

# define RLIM_NLIMITS	__KERNEL_RLIM_NLIMITS

# define RLIM_INFINITY	__KERNEL_RLIM_INFINITY

# define RUSAGE_SELF	__KERNEL_RUSAGE_SELF
# define RUSAGE_CHILDREN __KERNEL_RUSAGE_CHILDREN
# define RUSAGE_BOTH	__KERNEL_RUSAGE_BOTH

# define PRIO_MIN	__KERNEL_PRIO_MIN
# define PRIO_MAX	__KERNEL_PRIO_MAX

# define PRIO_PROCESS	__KERNEL_PRIO_PROCESS
# define PRIO_PGRP	__KERNEL_PRIO_PGRP
# define PRIO_USER	__KERNEL_PRIO_USER
#endif /* __KERNEL_STRICT_NAMES */

#endif /* _LINUX_RESOURCE_H */
