#ifndef _LINUX_SEM_H
#define _LINUX_SEM_H

#include <linux/ipc.h>
#include <asm/sembits.h>

#ifndef __KERNEL_STRICT_NAMES
# include <asm/seminfobits.h>
#endif

#define __KERNEL_SEMMNI	128		/* ?  max # of semaphore identifiers */
#define __KERNEL_SEMMSL	32		/* <= 512 max num of semaphores per id */
#define __KERNEL_SEMMNS	(__KERNEL_SEMMNI*__KERNEL_SEMMSL) /* ? max # of semaphores in system */
#define __KERNEL_SEMOPM	32		/* ~ 100 max num of ops per semop call */
#define __KERNEL_SEMVMX	32767		/* semaphore maximum value */

/* unused */
#define __KERNEL_SEMUME	__KERNEL_SEMOPM	/* max num of undo entries per process */
#define __KERNEL_SEMMNU	__KERNEL_SEMMNS	/* num of undo structures system wide */
#define __KERNEL_SEMAEM	(__KERNEL_SEMVMX >> 1)	  /* adjust on exit max value */
#define __KERNEL_SEMMAP	__KERNEL_SEMMNS	/* # of entries in semaphore map */
#define __KERNEL_SEMUSZ	20		/* sizeof struct sem_undo */

#ifndef __KERNEL_STRICT_NAMES
# define SEM_UNDO	__KERNEL_SEM_UNDO
# define GETPID		__KERNEL_GETPID
# define GETVAL		__KERNEL_GETVAL
# define GETALL		__KERNEL_GETALL
# define GETNCNT	__KERNEL_GETNCNT
# define GETZCNT	__KERNEL_GETZCNT
# define SETVAL		__KERNEL_SETVAL
# define SETALL		__KERNEL_SETALL
# define SEMMNI		__KERNEL_SEMMNI
# define SEMMSL		__KERNEL_SEMMSL
# define SEMMNS		__KERNEL_SEMMNS
# define SEMOPM		__KERNEL_SEMOPM
# define SEMVMX		__KERNEL_SEMVMX
# define SEMUME		__KERNEL_SEMUME
# define SEMMNU		__KERNEL_SEMMNU
# define SEMAEM		__KERNEL_SEMAEM
# define SEMMAP		__KERNEL_SEMMAP
# define SEMUSZ		__KERNEL_SEMUSZ
# define SEM_STAT	__KERNEL_SEM_STAT
# define SEM_INFO	__KERNEL_SEM_INFO
#endif

#ifdef __KERNEL__

/* One semaphore structure for each semaphore in the system. */
struct sem {
  short   semval;         /* current value */
  short   sempid;         /* pid of last operation */
};

/* One queue for each semaphore set in the system. */
struct sem_queue {
    struct sem_queue *	next;	 /* next entry in the queue */
    struct sem_queue **	prev;	 /* previous entry in the queue, *(q->prev) == q */
    struct wait_queue *	sleeper; /* sleeping process */
    struct sem_undo *	undo;	 /* undo structure */
    int    		pid;	 /* process id of requesting process */
    int    		status;	 /* completion status of operation */
    struct semid_ds *	sma;	 /* semaphore array for operations */
    struct sembuf *	sops;	 /* array of pending operations */
    int			nsops;	 /* number of operations */
};

/* Each task has a list of undo requests. They are executed automatically
 * when the process exits.
 */
struct sem_undo {
    struct sem_undo *  proc_next; /* next entry on this process */
    struct sem_undo *  id_next;	  /* next entry on this semaphore set */
    int		       semid;	  /* semaphore set identifier */
    short *	       semadj;	  /* array of adjustments, one per semaphore */
};

asmlinkage int sys_semget (key_t key, int nsems, int semflg);
asmlinkage int sys_semop (int semid, struct sembuf *sops, unsigned nsops);
asmlinkage int sys_semctl (int semid, int semnum, int cmd, union semun arg);

#endif /* __KERNEL__ */

#endif /* _LINUX_SEM_H */
